@@ -28,68 +28,68 @@ class DEFAULT_ORE(Enum):
 
 class VanillaEquipments(Enum):
 	""" Default vanilla equipments values (durability, armor, armor_toughness, knockback_resistance, attack_damage, attack_speed) """
-	HELMET			= {	DEFAULT_ORE.LEATHER:	{"durability": 55,		"generic.armor": 1},
-						DEFAULT_ORE.CHAINMAIL:	{"durability": 165,		"generic.armor": 2},
-						DEFAULT_ORE.IRON:		{"durability": 165,		"generic.armor": 2},
-						DEFAULT_ORE.GOLD:		{"durability": 77,		"generic.armor": 2},
-						DEFAULT_ORE.DIAMOND:	{"durability": 363,		"generic.armor": 3,	"generic.armor_toughness": 2},
-			 			DEFAULT_ORE.NETHERITE:	{"durability": 407,		"generic.armor": 3,	"generic.armor_toughness": 3,	"generic.knockback_resistance": 0.1}
+	HELMET			= {	DEFAULT_ORE.LEATHER:	{"durability": 55,		"armor": 1},
+						DEFAULT_ORE.CHAINMAIL:	{"durability": 165,		"armor": 2},
+						DEFAULT_ORE.IRON:		{"durability": 165,		"armor": 2},
+						DEFAULT_ORE.GOLD:		{"durability": 77,		"armor": 2},
+						DEFAULT_ORE.DIAMOND:	{"durability": 363,		"armor": 3,	"armor_toughness": 2},
+			 			DEFAULT_ORE.NETHERITE:	{"durability": 407,		"armor": 3,	"armor_toughness": 3,	"knockback_resistance": 0.1}
 					}
-	CHESTPLATE		= {	DEFAULT_ORE.LEATHER:	{"durability": 80,		"generic.armor": 3},
-						DEFAULT_ORE.CHAINMAIL:	{"durability": 240,		"generic.armor": 5},
-						DEFAULT_ORE.IRON:		{"durability": 240,		"generic.armor": 6},
-						DEFAULT_ORE.GOLD:		{"durability": 112,		"generic.armor": 5},
-						DEFAULT_ORE.DIAMOND: 	{"durability": 528,		"generic.armor": 8,	"generic.armor_toughness": 2},
-						DEFAULT_ORE.NETHERITE:	{"durability": 592,		"generic.armor": 8,	"generic.armor_toughness": 3,	"generic.knockback_resistance": 0.1}
+	CHESTPLATE		= {	DEFAULT_ORE.LEATHER:	{"durability": 80,		"armor": 3},
+						DEFAULT_ORE.CHAINMAIL:	{"durability": 240,		"armor": 5},
+						DEFAULT_ORE.IRON:		{"durability": 240,		"armor": 6},
+						DEFAULT_ORE.GOLD:		{"durability": 112,		"armor": 5},
+						DEFAULT_ORE.DIAMOND: 	{"durability": 528,		"armor": 8,	"armor_toughness": 2},
+						DEFAULT_ORE.NETHERITE:	{"durability": 592,		"armor": 8,	"armor_toughness": 3,	"knockback_resistance": 0.1}
 					}
-	LEGGINGS		= {	DEFAULT_ORE.LEATHER:	{"durability": 75,		"generic.armor": 2},
-  						DEFAULT_ORE.CHAINMAIL:	{"durability": 225,		"generic.armor": 4},
-						DEFAULT_ORE.IRON:		{"durability": 225,		"generic.armor": 5},
-						DEFAULT_ORE.GOLD:		{"durability": 105,		"generic.armor": 3},
-						DEFAULT_ORE.DIAMOND:	{"durability": 495,		"generic.armor": 6,	"generic.armor_toughness": 2},
-						DEFAULT_ORE.NETHERITE:	{"durability": 555,		"generic.armor": 6,	"generic.armor_toughness": 3,	"generic.knockback_resistance": 0.1}
+	LEGGINGS		= {	DEFAULT_ORE.LEATHER:	{"durability": 75,		"armor": 2},
+  						DEFAULT_ORE.CHAINMAIL:	{"durability": 225,		"armor": 4},
+						DEFAULT_ORE.IRON:		{"durability": 225,		"armor": 5},
+						DEFAULT_ORE.GOLD:		{"durability": 105,		"armor": 3},
+						DEFAULT_ORE.DIAMOND:	{"durability": 495,		"armor": 6,	"armor_toughness": 2},
+						DEFAULT_ORE.NETHERITE:	{"durability": 555,		"armor": 6,	"armor_toughness": 3,	"knockback_resistance": 0.1}
 					}
-	BOOTS			= {	DEFAULT_ORE.LEATHER:	{"durability": 65,		"generic.armor": 1},
-						DEFAULT_ORE.CHAINMAIL:	{"durability": 195,		"generic.armor": 1},
-						DEFAULT_ORE.IRON:		{"durability": 195,		"generic.armor": 2},
-						DEFAULT_ORE.GOLD:		{"durability": 95,		"generic.armor": 1},
-						DEFAULT_ORE.DIAMOND:	{"durability": 429,		"generic.armor": 3,	"generic.armor_toughness": 2},
-						DEFAULT_ORE.NETHERITE:	{"durability": 481,		"generic.armor": 3,	"generic.armor_toughness": 3,	"generic.knockback_resistance": 0.1}
+	BOOTS			= {	DEFAULT_ORE.LEATHER:	{"durability": 65,		"armor": 1},
+						DEFAULT_ORE.CHAINMAIL:	{"durability": 195,		"armor": 1},
+						DEFAULT_ORE.IRON:		{"durability": 195,		"armor": 2},
+						DEFAULT_ORE.GOLD:		{"durability": 95,		"armor": 1},
+						DEFAULT_ORE.DIAMOND:	{"durability": 429,		"armor": 3,	"armor_toughness": 2},
+						DEFAULT_ORE.NETHERITE:	{"durability": 481,		"armor": 3,	"armor_toughness": 3,	"knockback_resistance": 0.1}
 					}
-	SWORD			= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"generic.attack_damage": 4,		"generic.attack_speed": -2.50},
-						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"generic.attack_damage": 5,		"generic.attack_speed": -2.50},
-						DEFAULT_ORE.IRON:		{"durability": 250,		"generic.attack_damage": 6,		"generic.attack_speed": -2.50},
-						DEFAULT_ORE.GOLD:		{"durability": 32,		"generic.attack_damage": 4,		"generic.attack_speed": -2.50},
-						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"generic.attack_damage": 7,		"generic.attack_speed": -2.50},
-						DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"generic.attack_damage": 8,		"generic.attack_speed": -2.50}
+	SWORD			= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"attack_damage": 4,		"attack_speed": -2.50},
+						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"attack_damage": 5,		"attack_speed": -2.50},
+						DEFAULT_ORE.IRON:		{"durability": 250,		"attack_damage": 6,		"attack_speed": -2.50},
+						DEFAULT_ORE.GOLD:		{"durability": 32,		"attack_damage": 4,		"attack_speed": -2.50},
+						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"attack_damage": 7,		"attack_speed": -2.50},
+						DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"attack_damage": 8,		"attack_speed": -2.50}
 					}
-	PICKAXE			= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"generic.attack_damage": 2,		"generic.attack_speed": -2.75},
-						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"generic.attack_damage": 3,		"generic.attack_speed": -2.75},
-						DEFAULT_ORE.IRON:		{"durability": 250,		"generic.attack_damage": 4,		"generic.attack_speed": -2.75},
-						DEFAULT_ORE.GOLD:		{"durability": 32,		"generic.attack_damage": 2,		"generic.attack_speed": -2.75},
-						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"generic.attack_damage": 5,		"generic.attack_speed": -2.75},
-						DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"generic.attack_damage": 6,		"generic.attack_speed": -2.75}
+	PICKAXE			= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"attack_damage": 2,		"attack_speed": -2.75},
+						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"attack_damage": 3,		"attack_speed": -2.75},
+						DEFAULT_ORE.IRON:		{"durability": 250,		"attack_damage": 4,		"attack_speed": -2.75},
+						DEFAULT_ORE.GOLD:		{"durability": 32,		"attack_damage": 2,		"attack_speed": -2.75},
+						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"attack_damage": 5,		"attack_speed": -2.75},
+						DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"attack_damage": 6,		"attack_speed": -2.75}
 					}
-	AXE				= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"generic.attack_damage": 7,		"generic.attack_speed": -3.20},
-						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"generic.attack_damage": 9,		"generic.attack_speed": -3.20},
-						DEFAULT_ORE.IRON:		{"durability": 250,		"generic.attack_damage": 9,		"generic.attack_speed": -3.10},
-						DEFAULT_ORE.GOLD:		{"durability": 32,		"generic.attack_damage": 7,		"generic.attack_speed": -3.00},
-						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"generic.attack_damage": 9,		"generic.attack_speed": -3.00},
-						DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"generic.attack_damage": 10,	"generic.attack_speed": -3.00}
+	AXE				= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"attack_damage": 7,		"attack_speed": -3.20},
+						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"attack_damage": 9,		"attack_speed": -3.20},
+						DEFAULT_ORE.IRON:		{"durability": 250,		"attack_damage": 9,		"attack_speed": -3.10},
+						DEFAULT_ORE.GOLD:		{"durability": 32,		"attack_damage": 7,		"attack_speed": -3.00},
+						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"attack_damage": 9,		"attack_speed": -3.00},
+						DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"attack_damage": 10,	"attack_speed": -3.00}
 					}
-	SHOVEL			= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"generic.attack_damage": 2.5,	"generic.attack_speed": -3.20},
-						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"generic.attack_damage": 3.5,	"generic.attack_speed": -3.20},
-						DEFAULT_ORE.IRON:		{"durability": 250,		"generic.attack_damage": 4.5,	"generic.attack_speed": -3.10},
-						DEFAULT_ORE.GOLD:		{"durability": 32,		"generic.attack_damage": 2.5,	"generic.attack_speed": -3.00},
-						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"generic.attack_damage": 5.5,	"generic.attack_speed": -3.00},
-			 			DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"generic.attack_damage": 6.5,	"generic.attack_speed": -3.00}
+	SHOVEL			= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"attack_damage": 2.5,	"attack_speed": -3.20},
+						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"attack_damage": 3.5,	"attack_speed": -3.20},
+						DEFAULT_ORE.IRON:		{"durability": 250,		"attack_damage": 4.5,	"attack_speed": -3.10},
+						DEFAULT_ORE.GOLD:		{"durability": 32,		"attack_damage": 2.5,	"attack_speed": -3.00},
+						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"attack_damage": 5.5,	"attack_speed": -3.00},
+			 			DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"attack_damage": 6.5,	"attack_speed": -3.00}
 					}
-	HOE				= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"generic.attack_damage": 1,		"generic.attack_speed": -3.00},
-						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"generic.attack_damage": 1,		"generic.attack_speed": -2.00},
-						DEFAULT_ORE.IRON:		{"durability": 250,		"generic.attack_damage": 1,		"generic.attack_speed": -1.00},
-						DEFAULT_ORE.GOLD:		{"durability": 32,		"generic.attack_damage": 1,		"generic.attack_speed": -3.00},
-						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"generic.attack_damage": 1,		"generic.attack_speed": 0},
-						DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"generic.attack_damage": 1,		"generic.attack_speed": 0}
+	HOE				= {	DEFAULT_ORE.LEATHER:	{"durability": 59,		"attack_damage": 1,		"attack_speed": -3.00},
+						DEFAULT_ORE.CHAINMAIL:	{"durability": 131,		"attack_damage": 1,		"attack_speed": -2.00},
+						DEFAULT_ORE.IRON:		{"durability": 250,		"attack_damage": 1,		"attack_speed": -1.00},
+						DEFAULT_ORE.GOLD:		{"durability": 32,		"attack_damage": 1,		"attack_speed": -3.00},
+						DEFAULT_ORE.DIAMOND:	{"durability": 1561,	"attack_damage": 1,		"attack_speed": 0},
+						DEFAULT_ORE.NETHERITE:	{"durability": 2031,	"attack_damage": 1,		"attack_speed": 0}
 					}
 
 class EquipmentsConfig():
@@ -98,22 +98,28 @@ class EquipmentsConfig():
 		Args:
 			equivalent_to (DEFAULT_ORE):	The equivalent ore to compare to (ex: DEFAULT_ORE.DIAMOND)
 			pickaxe_durability (int):		The pickaxe durability that will be used to calculate the durability of other equipments
-			attributes (dict[str, float]):	(optional) Attributes with type "add_value" to add (not override) to the equipment (ex: "generic.attack_damage": 1.0, means 6 attack damage for diamond pickaxe)
-				{"generic.attack_damage": 1.0, "generic.armor": 1.0, "player.mining_efficiency": 0.1}
-				generic.attack_damage and player.mining_efficiency are always on tools
-				generic.armor and generic.armor_toughness is always on armor
+			attributes (dict[str, float]):	(optional) Attributes with type "add_value" to add (not override) to the equipment (ex: "attack_damage": 1.0, means 6 attack damage for diamond pickaxe)
+				{"attack_damage": 1.0, "armor": 1.0, "mining_efficiency": 0.1}
+				attack_damage and mining_efficiency are always on tools
+				armor and armor_toughness is always on armor
 		If you need a specific attribute for a generated item, you should append it afterward.
 		"""
 		self.equivalent_to = equivalent_to
 		self.pickaxe_durability = pickaxe_durability
 		self.attributes = attributes
+		for key in attributes.keys():
+			if "player." in key:
+				warning(f"Since 1.21.3, the 'player.' prefix is no longer written in attributes!!!")
+			elif "generic." in key:
+				warning(f"Since 1.21.3, the 'generic.' prefix is no longer written in attributes!!!")
+
 	def getter(self) -> tuple[DEFAULT_ORE, int, dict[str, float]]:
 		return self.equivalent_to, self.pickaxe_durability, self.attributes
 	def get_tools_attributes(self) -> dict[str, float]:
-		NOT_ON_TOOLS = ["generic.armor", "generic.armor_toughness"]
+		NOT_ON_TOOLS = ["armor", "armor_toughness"]
 		return {key: value for key, value in self.attributes.items() if key not in NOT_ON_TOOLS}
 	def get_armor_attributes(self) -> dict[str, float]:
-		NOT_ON_ARMOR = ["generic.attack_damage", "player.mining_efficiency"]
+		NOT_ON_ARMOR = ["attack_damage", "mining_efficiency"]
 		return {key: value for key, value in self.attributes.items() if key not in NOT_ON_ARMOR}
 
 # UUIDs utils
@@ -140,7 +146,7 @@ def format_attributes(config: dict, attributes: dict, slot: str, attr_config: di
 	return attribute_modifiers
 
 # Generate everything related to the ore
-def generate_everything_about_this_material(config: dict, database: dict[str, dict], material: str = "adamantium_fragment", equipments_config: EquipmentsConfig|None = EquipmentsConfig(DEFAULT_ORE.NETHERITE, 1873, {"attack_damage": 1.2, "player.mining_efficiency": 0.2})) -> list[str]:
+def generate_everything_about_this_material(config: dict, database: dict[str, dict], material: str = "adamantium_fragment", equipments_config: EquipmentsConfig|None = EquipmentsConfig(DEFAULT_ORE.NETHERITE, 1873, {"attack_damage": 1.2, "armor": 0.5, "mining_efficiency": 0.2})) -> None:
 	""" Generate everything related to the ore (armor, tools, weapons, ore, and ingredients (raw, nuggets, blocks)).
 		The function will try to find textures in the assets folder to each item
 		And return a list of generated items if you want to do something with it.
@@ -148,15 +154,14 @@ def generate_everything_about_this_material(config: dict, database: dict[str, di
 		database			(dict[str, dict]):		The database to update
 		material			(str):					The ore/material to generate everything about (ex: "adamantium_fragment", "steel_ingot", "minecraft:emerald", "minecraft:copper_ingot")
 		equipments_config	(EquipmentsConfig):	The base multiplier to apply
-	Returns:
-		list[str]:		The list of generated items (ex: ["adamantium_helmet", "raw_adamantium", "adamantium_block", ...])
 	"""
+	namespace: str = config['namespace']
 	# Constants
 	if '_' in material:
 		material_base = "_".join(material.split(":")[-1].split("_")[:-1])			# Get the base material name (ex: "adamantium" from "adamantium_fragment")
 	else:
 		material_base = material.split(":")[-1]										# Get the base material name (ex: "adamantium" from "adamantium_fragment")
-	main_ingredient = ingr_repr(material, config['namespace']) 						# Get the main ingredient for recipes
+	main_ingredient = ingr_repr(material, namespace) 						# Get the main ingredient for recipes
 	if equipments_config:
 		equivalent_to = equipments_config.equivalent_to
 		durability_factor = equipments_config.pickaxe_durability / VanillaEquipments.PICKAXE.value[equivalent_to]["durability"]
@@ -186,11 +191,28 @@ def generate_everything_about_this_material(config: dict, database: dict[str, di
 			database[block]["custom_data"]["smithed"]["dict"]["ore"] = {material_base: True}
 		if block.endswith("block"):
 			if block.startswith("raw") and f"raw_{material_base}.png" in config['textures_files']:
-				database[block][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"group":material_base,"category":"misc","shape":["XXX","XXX","XXX"],"ingredients":{"X":ingr_repr(f"raw_{material_base}", config['namespace'])}}]
+				database[block][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"group":material_base,"category":"misc","shape":["XXX","XXX","XXX"],"ingredients":{"X":ingr_repr(f"raw_{material_base}", namespace)}}]
 			else:
 				database[block][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"group":material_base,"category":"misc","shape":["XXX","XXX","XXX"],"ingredients":{"X":main_ingredient}}]
 		pass
 
+	# Armor equipment entity (top layer and leggings)
+	namespace_rp: str = f"{config['build_resource_pack']}/assets/{namespace}"
+	def handle_armor_layer(layer_num: int, gear_types: list[str], humanoid_type: str) -> bool:
+		layer_file: str = f"{material_base}_layer_{layer_num}.png"
+		if any(f"{material_base}_{gear}.png" in config['textures_files'] for gear in gear_types) and layer_file in config['textures_files']:
+			source: str = f"{config['assets_folder']}/textures/{layer_file}"
+			destination: str = f"{namespace_rp}/textures/entity/equipment/{humanoid_type}/{layer_file}"
+			super_copy(source, destination)
+			
+			model_file: str = f"{namespace_rp}/models/equipment/{material_base}.json"
+			model_data: dict = {"layers": {humanoid_type: [{"texture": f"{namespace}:{layer_file.replace('.png', '')}"}]}}
+			write_to_file(model_file, super_json_dump(model_data))
+			return True
+		return False
+	top_layer: bool = handle_armor_layer(1, ["helmet", "chestplate"], "humanoid")
+	bottom_layer: bool = handle_armor_layer(2, ["leggings", "boots"], "humanoid_leggings")
+
 	# Armor (helmet, chestplate, leggings, boots)
 	for gear in ["helmet", "chestplate", "leggings", "boots"]:
 		armor = material_base + "_" + gear
@@ -208,20 +230,28 @@ def generate_everything_about_this_material(config: dict, database: dict[str, di
 			database[armor][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"category":"equipment","shape":["XXX","X X"],"ingredients":{"X": main_ingredient}}]
 			gear_config = VanillaEquipments.HELMET.value[equivalent_to]
 			database[armor]["max_damage"] = int(gear_config["durability"] * durability_factor)
+			if top_layer:
+				database[armor]["equippable"] = {"slot":"head", "model":f"{namespace}:{material_base}"}
 		elif gear == "chestplate":
 			database[armor][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"category":"equipment","shape":["X X","XXX","XXX"],"ingredients":{"X": main_ingredient}}]
 			gear_config = VanillaEquipments.CHESTPLATE.value[equivalent_to]
 			database[armor]["max_damage"] = int(gear_config["durability"] * durability_factor)
+			if top_layer:
+				database[armor]["equippable"] = {"slot":"chest", "model":f"{namespace}:{material_base}"}
 		elif gear == "leggings":
 			database[armor][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"category":"equipment","shape":["XXX","X X","X X"],"ingredients":{"X": main_ingredient}}]
 			gear_config = VanillaEquipments.LEGGINGS.value[equivalent_to]
 			database[armor]["max_damage"] = int(gear_config["durability"] * durability_factor)
+			if bottom_layer:
+				database[armor]["equippable"] = {"slot":"legs", "model":f"{namespace}:{material_base}"}
 		elif gear == "boots":
 			database[armor][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"category":"equipment","shape":["X X","X X"],"ingredients":{"X": main_ingredient}}]
 			gear_config = VanillaEquipments.BOOTS.value[equivalent_to]
 			database[armor]["max_damage"] = int(gear_config["durability"] * durability_factor)
+			if bottom_layer:
+				database[armor]["equippable"] = {"slot":"feet", "model":f"{namespace}:{material_base}"}
 		database[armor]["attribute_modifiers"] = format_attributes(config, armor_attributes, SLOTS[gear], gear_config)
-	
+
 	# Tools (sword, pickaxe, axe, shovel, hoe)
 	for gear in ["sword", "pickaxe", "axe", "shovel", "hoe"]:
 		tool = material_base + "_" + gear
@@ -275,53 +305,51 @@ def generate_everything_about_this_material(config: dict, database: dict[str, di
 		database[item][USED_FOR_CRAFTING] = database[item].get(USED_FOR_CRAFTING, [])
 		if item.endswith("ingot") or item.endswith("fragment") or item == material_base:
 			if f"{material_base}_block.png" in config['textures_files']:
-				database[item][RESULT_OF_CRAFTING].append({"type":"crafting_shapeless","result_count":9,"category":"misc","group":material_base,"ingredients":[ingr_repr(f"{material_base}_block", config['namespace'])]})
+				database[item][RESULT_OF_CRAFTING].append({"type":"crafting_shapeless","result_count":9,"category":"misc","group":material_base,"ingredients":[ingr_repr(f"{material_base}_block", namespace)]})
 			if f"{material_base}_nugget.png" in config['textures_files']:
-				database[item][RESULT_OF_CRAFTING].append({"type":"crafting_shaped","result_count":1,"category":"misc","group":material_base,"shape":["XXX","XXX","XXX"],"ingredients":{"X":ingr_repr(f"{material_base}_nugget", config['namespace'])}})
+				database[item][RESULT_OF_CRAFTING].append({"type":"crafting_shaped","result_count":1,"category":"misc","group":material_base,"shape":["XXX","XXX","XXX"],"ingredients":{"X":ingr_repr(f"{material_base}_nugget", namespace)}})
 			if f"raw_{material_base}.png" in config['textures_files']:
-				database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"raw_{material_base}", config['namespace'])})
-				database[item][RESULT_OF_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"raw_{material_base}", config['namespace'])})
+				database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"raw_{material_base}", namespace)})
+				database[item][RESULT_OF_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"raw_{material_base}", namespace)})
 			if f"{material_base}_dust.png" in config['textures_files']:
-				database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"{material_base}_dust", config['namespace'])})
-				database[item][RESULT_OF_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"{material_base}_dust", config['namespace'])})
+				database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"{material_base}_dust", namespace)})
+				database[item][RESULT_OF_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"{material_base}_dust", namespace)})
 			if f"{material_base}_ore.png" in config['textures_files']:
-				database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"{material_base}_ore", config['namespace'])})
-				database[item][RESULT_OF_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"{material_base}_ore", config['namespace'])})
+				database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"{material_base}_ore", namespace)})
+				database[item][RESULT_OF_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"{material_base}_ore", namespace)})
 			if f"deepslate_{material_base}_ore.png" in config['textures_files']:
-				database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"deepslate_{material_base}_ore", config['namespace'])})
-				database[item][RESULT_OF_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"deepslate_{material_base}_ore", config['namespace'])})
+				database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"deepslate_{material_base}_ore", namespace)})
+				database[item][RESULT_OF_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"deepslate_{material_base}_ore", namespace)})
 		if item.endswith("dust"):
-			database[item][USED_FOR_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(item, config['namespace']),"result":main_ingredient})
-			database[item][USED_FOR_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(item, config['namespace']),"result":main_ingredient})
+			database[item][USED_FOR_CRAFTING].append({"type":"smelting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":200,"ingredient":ingr_repr(item, namespace),"result":main_ingredient})
+			database[item][USED_FOR_CRAFTING].append({"type":"blasting","result_count":1,"category":"misc","group":material_base,"experience":0.8,"cookingtime":100,"ingredient":ingr_repr(item, namespace),"result":main_ingredient})
 			database[item][RESULT_OF_CRAFTING].append({"type":PULVERIZING,"result_count":1,"category":"misc","group":material_base,"ingredient":main_ingredient})
 			for pulv_ingr in [f"raw_{material_base}",f"{material_base}_ore",f"deepslate_{material_base}_ore"]:
 				if f"{pulv_ingr}.png" in config['textures_files']:
-					database[item][RESULT_OF_CRAFTING].append({"type":PULVERIZING,"result_count":2,"category":"misc","group":material_base,"ingredient":ingr_repr(pulv_ingr, config['namespace'])})
+					database[item][RESULT_OF_CRAFTING].append({"type":PULVERIZING,"result_count":2,"category":"misc","group":material_base,"ingredient":ingr_repr(pulv_ingr, namespace)})
 		if item.endswith("nugget"):
 			database[item][RESULT_OF_CRAFTING].insert(0, {"type":"crafting_shapeless","result_count":9,"category":"misc","group":material_base,"ingredients":[main_ingredient]})
 			for gear in SLOTS.keys():
 				if f"{material_base}_{gear}.png" in config['textures_files']:
-					database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"equipment","experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"{material_base}_{gear}", config['namespace'])})
+					database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"equipment","experience":0.8,"cookingtime":200,"ingredient":ingr_repr(f"{material_base}_{gear}", namespace)})
 		if item.endswith("stick"):
 			database[item][RESULT_OF_CRAFTING].append({"type":"crafting_shaped","result_count":4,"category":"misc","shape":["X","X"],"ingredients":{"X":main_ingredient}})
 		if item.endswith("rod"):
 			database[item][RESULT_OF_CRAFTING].append({"type":"crafting_shaped","result_count":1,"category":"misc","shape":["X","X","X"],"ingredients":{"X":main_ingredient}})
 		pass
 
+
 # Generate everything about these ores
-def generate_everything_about_these_materials(config: dict, database: dict[str, dict], ores: dict[str, EquipmentsConfig|None]) -> dict[str, list[str]]:
+def generate_everything_about_these_materials(config: dict, database: dict[str, dict], ores: dict[str, EquipmentsConfig|None]) -> None:
 	""" Uses function 'generate_everything_about_this_material' for each ore in the ores dictionary.
 	Args:
 		config		(dict):				The configuration to use.
 		database	(dict[str, dict]):	The database to apply the ores to.
 		ores		(dict[str, EquipmentsConfig|None]):	The ores to apply.
-	Returns:
-		dict[str, list[str]]:	The list of generated items for each ore.
 	"""
-	generated_items = {}
 	for material, ore_config in ores.items():
-		generated_items[material] = generate_everything_about_this_material(config, database, material, ore_config)
-	return generated_items
+		generate_everything_about_this_material(config, database, material, ore_config)
+
 
 # Add recipes for dust
 def add_recipes_for_dust(config: dict, database: dict[str, dict], material: str, pulverize: list[str|dict], smelt_to: dict) -> None:
@@ -456,57 +484,21 @@ def deterministic_custom_model_data(config: dict, database: dict[str, dict], sta
 		starting_cmd	(int):				The starting custom model data.
 		blacklist		(list[str]):		The list of items to ignore.
 	"""
-	# Load cached custom model data
-	cached_custom_model_data = {}
-	if os.path.exists(config['cmd_cache']):
-		with open(config['cmd_cache'], "r") as f:
-			cached_custom_model_data = json.load(f)
-	
-	# For each item in the database, apply its cached custom model data if it exists
+	error("(deterministic_custom_model_data is deprecated since 1.21.3) Please now use the 'add_item_model_component' function instead.")
+
+# Add item model component
+def add_item_model_component(config: dict, database: dict[str, dict], black_list: list[str] = []) -> None:
+	""" Add an item model component to all items in the database.
+	Args:
+		config		(dict):				The configuration to get the namespace from.
+		database	(dict[str, dict]):	The database to add the item model component to.
+		black_list	(list[str]):		The list of items to ignore.
+	"""
+	namespace: str = config['namespace']
 	for item, data in database.items():
-		if item in cached_custom_model_data:
-			data["custom_model_data"] = cached_custom_model_data[item]
-
-	# Get maximum custom model data
-	max_cmd: int = starting_cmd - 1
-	for item, data in database.items():
-		if data.get("custom_model_data") and isinstance(data["custom_model_data"], int):
-			cmd: int = int(data["custom_model_data"])
-
-			# Add two custom model data if the item has an on/off texture
-			if any(item in texture and "_on" in texture for texture in config['textures_files']):
-				cmd += 1
-
-			# If it is a cake, add 6 custom model data for each slice
-			elif item.endswith("cake") and all(f"{item}_{face}.png" in config['textures_files'] for face in ["top","bottom","side","inner"]):
-				cmd += 7
-
-			# Increment the maximum custom model data
-			max_cmd = max(max_cmd, cmd)
-	
-	# For each item in the database, apply its new custom model data if it doesn't exist
-	for item, data in database.items():
-		if item not in black_list and (not data.get("custom_model_data") or not isinstance(data["custom_model_data"], int)):
-			data["custom_model_data"] = max_cmd + 1
-
-			# Add two custom model data if the item has an on/off texture
-			if any(item in texture and "_on" in texture for texture in config['textures_files']):
-				max_cmd += 2
-			
-			# If it is a cake, add 6 custom model data for each slice
-			elif item.endswith("cake") and all(f"{item}_{face}.png" in config['textures_files'] for face in ["top","bottom","side","inner"]):
-				max_cmd += 7
-			
-			# Else, add one custom model data
-			else:
-				max_cmd += 1
-	
-	# Update cache
-	for item, data in database.items():
-		if data.get("custom_model_data"):
-			cached_custom_model_data[item] = data["custom_model_data"]
-	with open(config['cmd_cache'], "w") as f:
-		super_json_dump(cached_custom_model_data, f)
+		if item in black_list:
+			continue
+		data["item_model"] = f"{namespace}:{item}"
 	return
 
 # Add item name and lore
@@ -605,7 +597,7 @@ class CustomOreGeneration():
 			debug("Found custom ore generation, adding 'smart_ore_generation' dependency")
 		self.check_validity()
 	
-	def check_validity(self) -> list[str]:
+	def check_validity(self) -> None:
 		""" Check if the custom ore generation configuration is valid """
 		if not isinstance(self.dimensions, list) or not all(isinstance(dimension, str) for dimension in self.dimensions):
 			error("Custom ore generation 'dimensions' must be a list of strings")
